"""
Module load handler for execution via python -m pytest_buildkite.
"""
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)


def main():
    """
    Test
    """
    print('Done.')


if __name__ == '__main__':
    main()
